package cn.com.poc.message.service.impl;


import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.knowledge.constant.KnowledgeConstant;
import cn.com.poc.knowledge.entity.BizKnowledgeDocumentEntity;
import cn.com.poc.knowledge.entity.BizKnowledgeTrainLogEntity;
import cn.com.poc.knowledge.entity.BizMemberKnowledgeRelationEntity;
import cn.com.poc.knowledge.service.BizKnowledgeDocumentService;
import cn.com.poc.knowledge.service.BizKnowledgeTrainLogService;
import cn.com.poc.knowledge.service.BizMemberKnowledgeRelationService;
import cn.com.poc.message.entity.KnowledgeTrainStatusMessage;
import cn.com.poc.message.entity.TrainKnowledgeMessage;
import cn.com.poc.message.service.KnowledgeConsumerService;
import cn.com.poc.message.service.KnowledgeProducerService;
import cn.com.poc.message.topic.KnowledgeTopic;
import cn.com.poc.thirdparty.resource.demand.ai.aggregate.DemandKnowledgeService;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.com.yict.framemax.tumbleweed.client.annotation.Consumer;
import org.apache.commons.collections4.MapUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;


@Service
public class KnowledgeConsumerServiceImpl implements KnowledgeConsumerService {

    final private Logger logger = LoggerFactory.getLogger(KnowledgeConsumerService.class);

    @Resource
    private BizKnowledgeTrainLogService bizKnowledgeTrainLogService;

    @Resource
    private BizKnowledgeDocumentService bizKnowledgeDocumentService;

    @Resource
    private DemandKnowledgeService demandKnowledgeService;

    @Resource
    private BizMemberKnowledgeRelationService bizMemberKnowledgeRelationService;

//    @Resource
//    private BizAiGcMemberUsageDetailsService bizAiGcMemberUsageDetailsService;
//
//    @Resource
//    private AiGcMemberFreqProducerService aiGcMemberFreqProducerService;

    @Resource
    private KnowledgeProducerService knowledgeProducerService;


    /**
     * 训练知识库
     *
     * @param message
     * @return
     */
    @Override
    @Consumer(topic = KnowledgeTopic.TRAIN_KNOWLEDGE, scale = 5, retry = false)
    public void trainKnowledge(TrainKnowledgeMessage message) throws Exception {
        //修改训练状态
        KnowledgeTrainStatusMessage trainStatusMessage = new KnowledgeTrainStatusMessage();
        trainStatusMessage.setStatus(KnowledgeConstant.TrainStatus.TRAINING);
        trainStatusMessage.setKdId(message.getKid());
        knowledgeProducerService.trainStatusUpdate(trainStatusMessage);
        try {
            String knowledgeId = demandKnowledgeService.trainKnowledge(message.getFileUrl());

            //训练日志
            BizKnowledgeTrainLogEntity bizKnowledgeTrainLogEntity = new BizKnowledgeTrainLogEntity();
            bizKnowledgeTrainLogEntity.setKdId(message.getKid());
            bizKnowledgeTrainLogEntity.setTimestamp(System.currentTimeMillis());
            bizKnowledgeTrainLogEntity.setTrainStatus(KnowledgeConstant.TrainStatus.COMPLETE);
            bizKnowledgeTrainLogEntity.setFailureLog("");
            bizKnowledgeTrainLogEntity.setIsDeleted(CommonConstant.IsDeleted.N);
            bizKnowledgeTrainLogService.save(bizKnowledgeTrainLogEntity);

            //训练完成
            KnowledgeTrainStatusMessage completeMessage = new KnowledgeTrainStatusMessage();
            completeMessage.setStatus(KnowledgeConstant.TrainStatus.COMPLETE);
            completeMessage.setKdId(message.getKid());
            completeMessage.setKnowledgeId(knowledgeId);
            knowledgeProducerService.trainStatusUpdate(completeMessage);
            //保存明细
//            saveUsageDetailByKnowledgeTrain(message.getKid());
        } catch (BusinessException e) {
            logger.warn("--------------message:{},知识库训练失败：{}------------", message, e.getMessage());

            //记录状态 训练失败
            trainStatusMessage.setStatus(KnowledgeConstant.TrainStatus.FAIL);
            trainStatusMessage.setKdId(message.getKid());
            knowledgeProducerService.trainStatusUpdate(trainStatusMessage);

            logger.warn("-------保存知识库训练失败状态----------");
            //训练日志
            BizKnowledgeTrainLogEntity bizKnowledgeTrainLogEntity = new BizKnowledgeTrainLogEntity();
            bizKnowledgeTrainLogEntity.setKdId(message.getKid());
            bizKnowledgeTrainLogEntity.setTimestamp(System.currentTimeMillis());
            bizKnowledgeTrainLogEntity.setTrainStatus(KnowledgeConstant.TrainStatus.FAIL);
            bizKnowledgeTrainLogEntity.setFailureLog(e.getMessage());
            bizKnowledgeTrainLogEntity.setIsDeleted(CommonConstant.IsDeleted.N);
            bizKnowledgeTrainLogService.save(bizKnowledgeTrainLogEntity);
            logger.warn("-------保存知识库训练失败日志----------");

            //次数回滚
//            if (MapUtils.isNotEmpty(message.getReduced())) {
//                logger.warn("----------知识库训练失败，训练次数回滚【开始】-------------");
//                AiGcMemberFreqMessage aiGcMemberFreqMessage = new AiGcMemberFreqMessage();
//                aiGcMemberFreqMessage.setReduceMap(message.getReduced());
//                aiGcMemberFreqProducerService.rollbackNum(aiGcMemberFreqMessage);
//                logger.warn("----------知识库训练失败，训练次数回滚【结束】-------------");
//            }
        }
    }


    @Override
    @Consumer(topic = KnowledgeTopic.TRAIN_STATUS, scale = 1, retry = true)
    public void trainStatusUpdate(KnowledgeTrainStatusMessage message) throws Exception {
        BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = new BizKnowledgeDocumentEntity();
        bizKnowledgeDocumentEntity.setTrainStatus(message.getStatus());
        bizKnowledgeDocumentEntity.setKnowledgeId(message.getKnowledgeId());
        bizKnowledgeDocumentService.update(message.getKdId(), bizKnowledgeDocumentEntity);
    }

    private void saveUsageDetailByKnowledgeTrain(Integer kdId) throws Exception {
        BizMemberKnowledgeRelationEntity bizMemberKnowledgeRelationEntity = new BizMemberKnowledgeRelationEntity();
        bizMemberKnowledgeRelationEntity.setMemberType("system");
        bizMemberKnowledgeRelationEntity.setKdId(kdId);
        bizMemberKnowledgeRelationEntity.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizMemberKnowledgeRelationEntity> entities = bizMemberKnowledgeRelationService.findByExample(bizMemberKnowledgeRelationEntity, null);
//        BizMemberKnowledgeRelationEntity memberKnowledgeRelationEntity = entities.get(0);
//
//        BizKnowledgeDocumentEntity bizKnowledgeDocumentEntity = bizKnowledgeDocumentService.get(kdId);
//        AiGcMemberUsageDetailsEntity aiGcMemberUsageDetailsEntity = new AiGcMemberUsageDetailsEntity();
//        aiGcMemberUsageDetailsEntity.setRelationId(memberKnowledgeRelationEntity.getMemberId());
//        aiGcMemberUsageDetailsEntity.setDomain(AiGcPaymentConfigurationConstant.DOMAIN.TRAIN);
//        aiGcMemberUsageDetailsEntity.setTitile(bizKnowledgeDocumentEntity.getDocumentName());
//        aiGcMemberUsageDetailsEntity.setDate(new Date());
//        aiGcMemberUsageDetailsEntity.setScenes(null);
//        aiGcMemberUsageDetailsEntity.setDetailType("知识库训练");
//        aiGcMemberUsageDetailsEntity.setOperation(AiGcMemberUsageDetailsConstant.Operation.USE);
//        aiGcMemberUsageDetailsEntity.setNum(bizKnowledgeDocumentEntity.getCharCount().intValue());
//        aiGcMemberUsageDetailsEntity.setIsDeleted(MkpCommonConstant.IsDeleted.N);
//        bizAiGcMemberUsageDetailsService.save(aiGcMemberUsageDetailsEntity);
    }

}
