package cn.com.poc.thirdparty.resource.demand.ai.aggregate.impl;

import cn.com.poc.thirdparty.resource.demand.ai.aggregate.DemandKnowledgeService;
import cn.com.poc.thirdparty.resource.demand.ai.entity.knowledge.*;
import cn.com.poc.thirdparty.resource.demand.member.service.DemandAuthService;
import cn.com.poc.support.dgTools.DgtoolsAbstractHttpClient;
import cn.com.poc.support.dgTools.constants.DgtoolsApiConstants;
import cn.com.poc.support.dgTools.result.AbstractResult;
import cn.com.yict.framemax.core.exception.BusinessException;
import cn.hutool.core.lang.Assert;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.http.Header;
import org.apache.http.message.BasicHeader;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Service
public class DemandKnowledgeServiceImpl implements DemandKnowledgeService {

    @Resource
    private DgtoolsAbstractHttpClient dgToolsAbstractHttpClient;

    @Resource
    private DemandAuthService demandAuthService;

    @Override
    public String trainKnowledge(String fileURL) {
        Assert.notBlank(fileURL);
        TrainKnowledgeRequest request = new TrainKnowledgeRequest();
        request.setDocumentUrl(fileURL);
        TrainKnowledgeResult trainKnowledgeResult = dgToolsAbstractHttpClient.doRequest(DgtoolsApiConstants.DgtoolsAI.TRAIN_KNOWLEDGE, request, getHeaders());
        if (null == trainKnowledgeResult) {
            throw new BusinessException("train knowledge error");
        }
        return trainKnowledgeResult.getKnowledgeId();
    }

    @Override
    public void delKnowledge(String knowledgeId) {
        Assert.notBlank(knowledgeId);
        DelKnowledgeRequest request = new DelKnowledgeRequest();
        request.setKnowledgeId(knowledgeId);
        AbstractResult abstractResult = dgToolsAbstractHttpClient.doRequest(DgtoolsApiConstants.DgtoolsAI.DEL_KNOWLEDGE, request, getHeaders());
        if (null == abstractResult) {
            throw new BusinessException("del knowledge error");
        }
    }

    @Override
    public List<String> searchKnowledge(String query, List<String> knowledgeIds, Integer topK) {
        if (CollectionUtils.isEmpty(knowledgeIds)) {
            return new ArrayList<String>();
        }

        Assert.notBlank(query);
        Assert.notNull(topK);

        SearchKnowledgeRequest searchKnowledgeRequest = new SearchKnowledgeRequest();
        searchKnowledgeRequest.setQuery(query);
        searchKnowledgeRequest.setKnowLedgeIds(knowledgeIds);
        searchKnowledgeRequest.setTopK(topK);
        SearchKnowledgeResult searchKnowledgeResult = dgToolsAbstractHttpClient.doRequest(DgtoolsApiConstants.DgtoolsAI.SEARCH_KNOWLEDGE, searchKnowledgeRequest, getHeaders());
        if (null == searchKnowledgeResult) {
            throw new BusinessException("search knowledge is error");
        }
        return searchKnowledgeResult.getDocuments();
    }

    private List<Header> getHeaders() {
        List<Header> headers = new ArrayList<>();
        headers.add(DgtoolsApiConstants.JSON_HEADER);
        headers.add(DgtoolsApiConstants.AI_HEADER);
        headers.add(new BasicHeader(DgtoolsApiConstants.HEADER_X_PLATFORM_AUTHORIZATION, demandAuthService.getToken()));
        return headers;
    }
}
